/****************************************************************************
 *
 * Project: -----
 *
 * Copyright by Olimex Ltd. All rights reserved.
 *
 * File: gsm.c
 * Description: Very simple GSM AT command handler.
 * Developer: Dimitar Dimitrov ( dinuxbg,gmail.com )
 *
 * Last change: $Date: 2009-04-30 17:25:54 +0300 (Thu, 30 Apr 2009) $
 * Revision: $Revision: 29 $
 * Id: $Id: gsm.c 692 2009-04-30 14:25:54Z dimitar $
 *
 ****************************************************************************/
#include <assert.h>
#include <math.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <stdlib.h>
#include <string.h>
#include <intrinsics.h>
#include "stm32f10x_lib.h"
#include "platform_config.h"

#include "utils.h"
#include "lcd-320x240.h"
#include "touchscreen.h"
#include "gps.h"

#include "gsm-hw.h"
#include "gsm.h"

/**
 * @file
 * Very simple GSM AT command handler.
 *
 *
 *
 */

static bool gsm_is_sane;

#define ASCII_SIB	0x1a
#define ASCII_CTRLZ	(ASCII_SIB)
#define ASCII_ESC	0x1b

const char *gsm_delim = ",";


static int str_starts_with(const char *str, const char *start)
{
	return !strncmp(str, start, strlen(start));
}

static char *gsm_get_line(unsigned int timeout_ms)
{
	char *p;
	unsigned int t0 = counter_1ms;
	
	do {
		p = gsm_hw_get_line();
		if(p && !strlen(p)) {
			/* filter out empty lines */
			gsm_hw_dealloc_line();
			p = 0;
		}
	} while(!p && ((counter_1ms - t0) < timeout_ms));
	return p;
}


static void parse_clip(char *line)
{
	char *tk, *phonenum;

	phonenum = strnext(&line, gsm_delim);
	if(!phonenum)
		return;
	while(*phonenum && isspace(*phonenum))
		phonenum++;

	/* type */
	tk = strnext(&line, gsm_delim);
	/* empty string */
	tk = strnext(&line, gsm_delim);
	/* empty */
	tk = strnext(&line, gsm_delim);
	/* alpha id */
	tk = strnext(&line, gsm_delim);

	/* CLI validity */
	tk = line;
	if(tk && !strcmp(tk,"0")) {
		size_t n;
		if(phonenum[0] == '"')
			phonenum++;
		n = strlen(phonenum);
		if(n && (phonenum[n-1] == '"')) {
			phonenum[n-1] = '\0';
		}
		if(gsm_callbacks.clip)
			gsm_callbacks.clip(phonenum);
	}
}


static void parse_unsolicited_line(char *line)
{
	assert(line);
	
	if(!strcmp(line,"RING")) {
		if(gsm_callbacks.ring)
			gsm_callbacks.ring();
	} else if(str_starts_with(line,"+CLIP:")) {
		line += strlen("+CLIP:");
		parse_clip(line);
	}
}

static int process_request(const char *req, const char *param, char **resp)
{
	char *p;
	char buf[128];
	int ntries;

	assert((strlen(req)+strlen(param)) < (sizeof(buf)-16));
	sprintf(buf, "AT%s%s", req, param);
	gsm_hw_send_line(buf);
	
	sprintf(buf,"%s:", req);
	for(ntries=8; ntries; ntries--) {
		p = gsm_get_line(500);
		if(!p) {
			ntries=-1;	/* terminate because of timeout */
			break;
		}
		if(!strcmp(p,"ERROR")) {
			return -EINVAL;
		}
		if(!resp && !strcmp(p,"OK")) {
			gsm_hw_dealloc_line();
			break;
		} else if(resp && str_starts_with(p,buf)) {
			*resp = p + strlen(buf);
			break;
		} else if(str_starts_with(p,"+CME ERROR:")) {
			/* Command error. */
			return -EINVAL;
			/*break;*/
		} else if(str_starts_with(p,"+CMS ERROR:")) {
			/* Command error. */
			return -EINVAL;
			/*break;*/
		} else if(str_starts_with(p,"+CMS ERROR:")) {
			/* Command error. */
			return -EINVAL;
			/*break;*/
		} else if(p) {
			parse_unsolicited_line(p);
			gsm_hw_dealloc_line();
		}
	}
	if(ntries<0) {
		return -EINVAL;
	} else {
		return 0;
	}
}


void gsm_poll(void)
{
	char *p;
	
	do {
		p = gsm_hw_get_line();
		if(p) {
			parse_unsolicited_line(p);
			gsm_hw_dealloc_line();
		}
	} while(p);
}


static int gsm_initiate_connection(void)
{
	char *p;
	int ntry;
	int stat;

	/* just in case hardware is in text input mode */
	//gsm_hw_send_byte(ASCII_ESC);
	
	for(ntry=8; ntry>=0; ntry--) {
		gsm_hw_send_line("AT");
		p = gsm_get_line(500);
		if(!p)
			continue;
		if(!strcmp(p,"AT")) {		/* check for echo */
			gsm_hw_dealloc_line();
			p = gsm_get_line(500);
			if(!p)
				continue;
		}
		if(!strcmp(p,"OK")) {
			gsm_hw_dealloc_line();
			break;
		} else {
			gsm_hw_dealloc_line();
		}
	}
	if(ntry < 0) {
		stat = -EINVAL;
	} else {
		stat = 0;
		/* disable echo */
		gsm_hw_send_line("ATE0");
		p = gsm_get_line(500);
		if(p && !strcmp(p,"ATE0")) {
			gsm_hw_dealloc_line();
		}
	}
	
	/* enable CLIP when ringing (ignore possible errors) */
	if(!stat)  {
		stat = process_request("+CLIP", "=1", 0);
	}
	/* set GSM character set */
	if(!stat) {
		stat = process_request("+CSCS", "=\"GSM\"", 0);
	}
	/* set SMS format to plain text */
	if(!stat) {
		stat = process_request("+CMGF", "=1", 0);
	}
	return stat;
}


void gsm_init(void)
{
	char *resp;

	gsm_hw_init_pwrkey();       // turn on device
	gsm_hw_init(115200);	
	gsm_is_sane = !gsm_initiate_connection();
	
	// try to communicate at higher baudrate
	if(!gsm_is_sane) {
		gsm_hw_init(115200 * 4);	
		gsm_is_sane = !gsm_initiate_connection();
		
		// if working at higher baud then reduce it!
		if(gsm_is_sane) {
			gsm_hw_send_line("AT+IPR=115200");
			resp = gsm_get_line(500);
			if( resp && !strcmp(resp, "OK")) {
				// switch to 115200 was sucessful, change UART to new baud as well
				gsm_hw_init(115200);
				counter_1ms = 0;
				while(counter_1ms < 1000);
				gsm_is_sane = !gsm_initiate_connection();
				
				// now save configuration to non volatile memory
				if(gsm_is_sane) {
					gsm_hw_send_line("AT&W");
					resp = gsm_get_line(1500);
					if( resp && !strcmp(resp, "OK") ) {
						// stored sucessfully
						LCD_ConSetPos(0,0);
						LCD_Clear(LCD_COLOR_BLACK);
						LCD_Printf("BAUD CHANGED TO:\n   115200bps");
						counter_1ms = 0;
						while(counter_1ms < 1500);
						return;
					}
				}
			}
			
			// display the error message
			LCD_ConSetPos(0,0);
			LCD_Clear(LCD_COLOR_RED);
			LCD_Printf("ERROR CHANGING\n   BAUDRATE");
			gsm_is_sane = 0;
			counter_1ms = 0;
			while(counter_1ms < 1500);			
		}
	}
 }

int gsm_check_sanity(void)
{
	if(!gsm_is_sane)
		gsm_is_sane = !gsm_initiate_connection();
	return gsm_is_sane;
}

int gsm_get_battery_perc(struct gsm_battery_status *st)
{
	int stat;
	char *resp, *tk;

	stat = process_request("+CBC","",&resp);
	if(!resp)
		return -EINVAL;
	if(!stat) {
		tk = strnext(&resp, gsm_delim);
		if(!tk)
			stat = -EINVAL;
		else
			st->bcs = atoi(tk);
	}

	if(!stat) {
		tk = strnext(&resp, gsm_delim);
		if(!tk)
			stat = -EINVAL;
		else
			st->bcl = atoi(tk);
	}

	if(!stat) {
		tk = resp;
		if(!tk)
			stat = -EINVAL;
		else
			st->voltage_mv = atoi(tk);
	}

	if(!stat) {
		gsm_hw_dealloc_line();
	}

	return stat;
}


int gsm_get_signal_quality_report(void)
{
  int stat = -1;
  char *resp;
  int ntries;
  char *p;

  gsm_hw_send_line("AT+CSQ");

  for(ntries=2; ntries; ntries--) {
    p = gsm_get_line(500);
    LCD_Printf("S QLTY = %s \n", p);
    if(!p) {
            ntries=-1;	/* terminate because of timeout */
            break;
    }
    if(!strcmp(p,"ERROR")) {
            return -EINVAL;
    }
    if(!resp && !strcmp(p,"OK")) {
            gsm_hw_dealloc_line();
//            break;
    }
    else if(str_starts_with(p,"+CME ERROR:")) {
            /* Command error. */
            return -EINVAL;
            /*break;*/
    } else if(str_starts_with(p,"+CMS ERROR:")) {
            /* Command error. */
            return -EINVAL;
            /*break;*/
    } else if(str_starts_with(p,"+CMS ERROR:")) {
            /* Command error. */
            return -EINVAL;
            /*break;*/
    } else if(p) {
            parse_unsolicited_line(p);
            gsm_hw_dealloc_line();
    }
  }

	

  if(!stat)
  {
	gsm_hw_dealloc_line();
  }

	return stat;

}

int gsm_send_sms(const char *phonenum, const char *text)
{
	char buf[64];
	char *resp;
	int ntry;

	sprintf(buf, "AT+CMGS=\"%s\"", phonenum);
	gsm_hw_send_line(buf);

#if 0
	for(ntry=8; ntry>=0; ntry--) {
		resp = gsm_get_line(500);
		if(!resp) {
			ntry = -1;
			break;
		}
		if(!strcmp(resp,">")) {
			gsm_hw_dealloc_line();
			break;
		} else {
			parse_unsolicited_line(resp);
			gsm_hw_dealloc_line();
		}
	}
	if(ntry<0)
		return -EINVAL;
#endif
	gsm_hw_send_line(text);
	gsm_hw_send_byte(ASCII_SIB);

	for(ntry=8; ntry>=0; ntry--) {
		resp = gsm_get_line(500);
		if(!resp) {
			ntry = -1;
			break;
		}
		if(!strcmp(resp,"OK")) {
			gsm_hw_dealloc_line();
			break;
		} else {
			parse_unsolicited_line(resp);
			gsm_hw_dealloc_line();
		}
	}
	if(ntry<0)
		return -EINVAL;
	return 0;
}


